#!/bin/bash

## 
## Usage: %PROG% FILENAME.asm
## 
##   Assembles FILENAME.asm then runs the result through
##   the 6502 emulator at memory location '%STARTUP%'.
## 
## Options:
## 
##   -s ADDRESS   Give a non-default load/startup address.
##   -q           Be quiet (don't display progress messages).
##   -h           Show this usage message, then exit.
## 
## Example:
## 
##   $ %PROG% tests/fibonacci.asm
## 
# -----
#
#   Copyright (C) 2020 Timothy Richard Musson
#
#   Copying and distribution of this file, with or without modification,
#   are permitted provided the copyright notice and this notice are
#   preserved.
#
# 2020-05-04

PROG=$(basename $0)

PREFIX=./src
ASSEMBLER=$PREFIX/asm65
EMULATOR=$PREFIX/emu65

MACHINE=6502
STARTUP=0x1000
RAM_SIZE=65535

TMPFILE=${TMPDIR:-/tmp}/$$.$PROG
trap 'rm -f "$TMPFILE"' EXIT


invocationError ()
{
  echo "Try '$PROG -h'" >&2
  exit 1
}


showUsage ()
{
  sed -n '/^##/s/^## //p' $0 \
    | sed "s#%PROG%#${PROG}#g;s#%STARTUP%#${STARTUP}#g"
  exit
}


while getopts 'hqs:' Option
do
  case $Option in
  q) QUIET=1 ;;
  s) STARTUP=$OPTARG ;;
  h) showUsage ;;
  *) invocationError ;;
  esac
done
shift $(($OPTIND - 1))

[ "$#" -ne 1 ] && invocationError


SOURCE_ASM=$1

$ASSEMBLER --hex               \
           --origin $STARTUP   \
           --counter           \
           ${QUIET:+--quiet}   \
           --output "$TMPFILE" \
           "$SOURCE_ASM"       \
  && $EMULATOR --machine machines/$MACHINE     \
               --ram $RAM_SIZE                 \
               --load "${TMPFILE}:${STARTUP}"  \
               ${QUIET:+--quiet}               \
               --startup $STARTUP



